unit uDecisionMatrix;

interface

uses
  Classes;

type
  TDecisionMatrix = class(TObject)
  private
    FDefaultResult: boolean;
    FMatrix: TStringList;
    FListOfLists: TList;

    function YValueIndex(const Value: string): integer;
    function XValueIndex(const Value: string): integer;
    function AddXValue(const Value: string): integer;
    function AddYValue(const Value: string): integer;
    function NewStringList: TStringList;
  public
    constructor Create;
    destructor Destroy;
    function CheckResult(const XValue, YValue: string): boolean;
    procedure SetResult(const XValue, YValue: string;
                        const ResultValue: boolean);

    property DefaultResult: boolean read FDefaultResult
                                    write FDefaultResult;
  end;

implementation

{ TDecisionMatrix }

constructor TDecisionMatrix.Create;
begin
  inherited Create;

  FListOfLists := TList.Create;

  FMatrix := NewStringList;                        // Ensure we have an X axis
  FMatrix.Objects[0] := NewStringList;             // Ensure we have a Y axis

  DefaultResult := False;                          // Set default for the default!
end;

destructor TDecisionMatrix.Destroy;
var
  I: integer;
begin
  if FListOfLists <> nil then
  begin
    for I := 0 to (FListOfLists.Count - 1) do
      if FListOfLists.Items[I] <> nil then
        TObject(FListOfLists.Items[I]).Free;
    FListOfLists.Free;
  end;

  inherited Destroy;
end;

procedure TDecisionMatrix.SetResult(const XValue, YValue: string;
                                    const ResultValue: boolean);
var
  i: integer;
  XIndex: integer;
  YIndex: integer;
  TextResultValue: string;
begin
  case ResultValue of
    False: TextResultValue := 'N';
    True:  TextResultValue := 'Y';
  end;

  XIndex := XValueIndex(XValue);
  if XIndex = 0 then
    XIndex := AddXValue(XValue);

  YIndex := YValueIndex(YValue);
  if YIndex = 0 then
    YIndex := AddYValue(YValue);

  with TStringList(FMatrix.Objects[XIndex]) do
  begin
    if YIndex >= Count then
      for I := Count to YIndex do
        Add('');
    Strings[YIndex] := TextResultValue;
  end;
end;

function TDecisionMatrix.CheckResult(const XValue, YValue: string): boolean;
var
  XIndex: integer;
  YIndex: integer;
  TextResultValue: string;
begin
  Result := DefaultResult;

  XIndex := XValueIndex(XValue);
  YIndex := YValueIndex(YValue);

  if (XIndex < FMatrix.Count) and
     (YIndex < TStringList(FMatrix.Objects[XIndex]).Count) then
  begin
    TextResultValue := TStringList(FMatrix.Objects[XIndex]).Strings[YIndex];
    Result := TextResultValue = 'Y';
  end;
end;

function TDecisionMatrix.AddXValue(const Value: string): integer;
begin
  Result := FMatrix.Add(Value);
  FMatrix.Objects[Result] := NewStringList;
end;

function TDecisionMatrix.AddYValue(const Value: string): integer;
begin
  Result := TStringList(FMatrix.Objects[0]).Add(Value);
end;

function TDecisionMatrix.XValueIndex(const Value: string): integer;
var
  X: integer;
begin
  X := 0;
  Result := 0;
  with FMatrix do
  begin
    while X < Count do
      if Strings[X] = Value then
      begin
        Result := X;
        X := Count;                      // Don't look any further
      end
      else
        inc(X);
  end;
end;

function TDecisionMatrix.YValueIndex(const Value: string): integer;
var
  Y: integer;
begin
  Y := 0;
  Result := 0;

  with TStringList(FMatrix.Objects[0]) do
  begin
    while Y < Count do
      if Strings[Y] = Value then
      begin
        Result := Y;
        Y := Count;                      // Don't look any further
      end
      else
        inc(Y);
  end;
end;

function TDecisionMatrix.NewStringList: TStringList;
begin
  Result := TStringList.Create;
  TStringList(Result).Add('');           // Initialise new list
  FListOfLists.Add(@Result);
end;

end.
